import os
import requests
import yaml


def get_chat_id(bot_token, chat_name):
    message = "trying to find out chat id"
    url = f"https://api.telegram.org/bot{bot_token}/sendMessage?chat_id=@{chat_name}&text={message}"

    res = requests.get(url).json()

    return res['result']['chat']['id']

def create_credentials(bot_token, chat_name, PATH_TO_SAVE=None):
    chat_id = get_chat_id(bot_token=bot_token, chat_name=chat_name)

    credentials = {"TOKEN" : bot_token,
                   "CHAT_ID" : chat_id,
                   "CHAT_NAME" : chat_name}
    
    if PATH_TO_SAVE:
        with open(PATH_TO_SAVE, 'w') as write_credentials:
            yaml.dump(credentials, write_credentials)

    return credentials


import yaml
import cv2

import telebot
from telebot.types import InputMediaPhoto



class TGSummaryWriterBase():
    def __init__(self, path_to_credentials, experiment_name):
        self.credentials = self.load_credentials(path_to_credentials)
        self.experiment_name = experiment_name
        self.bot = telebot.TeleBot(self.credentials['TOKEN'],
                                   parse_mode=None)
        self.topic = self.bot.create_forum_topic(self.credentials['CHAT_ID'], 
                                                 self.experiment_name)
        self.delimiter = "-"*90

    def send_message(self, message):
        self.bot.send_message(self.credentials['CHAT_ID'], text=message, 
                              reply_to_message_id=self.topic.message_thread_id)
        
    def send_pandas_df(self, df):
        message = f"<pre>\n{df.to_markdown()}\n</pre>"
        self.bot.send_message(self.credentials["CHAT_ID"], text=message,
                              reply_to_message_id=self.topic.message_thread_id, parse_mode='HTML')
        
    def send_numpy_image(self, np_image):
        success, img_enc = cv2.imencode('.jpg', np_image)
        assert(success)
        
        img_enc = img_enc.tostring()

        self.bot.send_photo(self.credentials['CHAT_ID'], img_enc,
                            reply_to_message_id=self.topic.message_thread_id)
    
    def send_numpy_images(self, np_images):
        encoded_imgs = []
        for img in np_images:
            success, img_enc = cv2.imencode('.jpg', img)
            assert(success)
            
            img_enc = img_enc.tostring()
            encoded_imgs.append(InputMediaPhoto(img_enc))
        
        self.bot.send_media_group(self.credentials['CHAT_ID'], encoded_imgs, 
                                  reply_to_message_id=self.topic.message_thread_id)
    

    def send_delimiter(self):
        self.send_message(self.delimiter)

    @staticmethod
    def load_credentials(path_to_cred):
        with open(path_to_cred, 'r') as read_c:
            credentials = yaml.safe_load(read_c)
        return credentials




import pandas as pd
from .tg_bot import TGSummaryWriterBase


class TGTableSummaryWriter(TGSummaryWriterBase):
    def __init__(self, path_to_credentials, experiment_name):
        super().__init__(path_to_credentials, experiment_name)

        self.records = pd.DataFrame()
        self.description = None
        self.send_message(self.experiment_name)

    def add_description(self, text_message, **kwargs):
        self.description = f"{text_message}\n"
        for desc_name in kwargs:
            desc = kwargs[desc_name]
            self.description += f"{desc_name} : {desc}\n"

    def add_record(self, **kwargs):
        df_new_record = pd.DataFrame.from_records(kwargs, index=[0])
        self.records = pd.concat([self.records, df_new_record], ignore_index=True)

    def send(self, sort_by=None, ascending=None, round=None):
        tmp_records = self.records

        if sort_by is not None and ascending is not None:
            tmp_records = self.records.sort_values(by=sort_by, ascending=ascending)
        elif sort_by is not None:
            tmp_records = self.records.sort_values(by=sort_by)
        
        if round:
            tmp_records_floats = tmp_records.select_dtypes(include=[float])
            tmp_records[tmp_records_floats.columns] = tmp_records_floats.round(round)
        print(tmp_records)
        self.send_delimiter()
        self.send_pandas_df(tmp_records)


        


from collections import defaultdict

import pandas as pd
import numpy as np

import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt

from .tg_bot import TGSummaryWriterBase

class TGImgSummaryWriter(TGSummaryWriterBase):
    def __init__(self,  path_to_credentials, experiment_name):
        super().__init__(path_to_credentials, experiment_name)

        self.metrics = {}        
        self.images = defaultdict(lambda : list())
        self.description = None

    def add_image(self, np_img, group=None):
        self.images[group].append(np_img)
    
    def add_description(self, description):
        self.description = description

    def add_scalar(self, name, value, step, group=None):
            if (name, group) in self.metrics:
                self.metrics[(name, group)][0].append(value)
                self.metrics[(name, group)][1].append(step)
            else:
                self.metrics[(name, group)] = [[value], [step]]

    def collect_statistics(self):
        metrics_statistics = []
        for name, group in self.metrics:
            metrics_data = self.metrics[(name, group)]

            cur_metric_stats = {"metric" : name}
            cur_metric_stats['mean'] = np.mean(metrics_data[0])
            cur_metric_stats['max'] = np.max(metrics_data[0])
            cur_metric_stats['min'] = np.min(metrics_data[0])
            cur_metric_stats['argmax'] = metrics_data[0][np.argmax(metrics_data[0])]
            cur_metric_stats['argmin'] = metrics_data[0][np.argmin(metrics_data[0])]
            cur_metric_stats['mean -5:'] = np.mean(metrics_data[0][-5:])

            metrics_statistics.append(cur_metric_stats)

        return pd.DataFrame.from_records(metrics_statistics)
    
    def collect_images(self, figsize=(10, 8)):
        stats_imgs = defaultdict(lambda : list())
        
        for name, group in self.metrics:
            metrics_data = self.metrics[(name, group)]
            fig, ax = plt.subplots()
            ax.set_title(f'Record name : {name}')
            ax.set_xlabel('step')
            ax.set_ylabel(name)
            ax.plot(metrics_data[1], metrics_data[0], 'g', label=name)

            ax.legend()

            fig.canvas.draw()
            data = np.frombuffer(fig.canvas.tostring_rgb(), dtype=np.uint8)
            data = data.reshape(fig.canvas.get_width_height()[::-1] + (3,))

            stats_imgs[group].append(data)
            plt.close(fig)
        
        return stats_imgs
    
    def send(self, send_statistics=True, send_images=True, send_plots=True):
        self.send_delimiter()
        self.send_message(self.description)
        if send_statistics:
            statistics = self.collect_statistics()
            self.send_pandas_df(statistics)
        if send_plots:
            plots = self.collect_images()
            for group_name in plots:
                group_plots = plots[group_name]
                self.send_numpy_images(group_plots)
        if send_images:
            for group_name in self.images:
                group_images = self.images[group_name]
                self.send_numpy_images(group_images)

        self.images = defaultdict(lambda : list())
    





        
    





from distutils.core import setup
setup(
  name = 'tg_bot_ml',
  packages = ['tg_bot_ml'],
  version = '0.0.1',
  license='GPLv3',
  description = 'Simple telegram bot for logging ML statistics.',
  author = 'Dmitry',
  author_email = 'michalych2014@yandex.ru',
  url = 'https://github.com/DmitryAsdre/telegram-ml-bot-stats.git', 
  download_url = 'https://github.com/DmitryAsdre/telegram-ml-bot-stats/archive/refs/heads/main.zip',
  keywords = ['Telegram', 'Bot', 'ML', 'Machine Learning', 'Logger'],
  install_requires=[
          'pandas',
          'numpy',
          'matplotlib',
          'pyTelegramBotAPI',
          'yaml',
          'opencv-python', 
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',  
    'Programming Language :: Python :: 3',      
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8', 
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3.10',
    'Programming Language :: Python :: 3.11'
  ],
)